#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: NoiseCancelling_v1
# Author: PA0SIM
# Description: Noise Cancelling using two identical phase coherent receivers
# GNU Radio version: 3.8.2.0

from distutils.version import StrictVersion

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print("Warning: failed to XInitThreads()")

import os
import sys
sys.path.append(os.environ.get('GRC_HIER_PATH', os.path.expanduser('~/.grc_gnuradio')))

from ABXO import ABXO  # grc-generated hier_block
from Bulk_Space_v1 import Bulk_Space_v1  # grc-generated hier_block
from Bulk_space_plot_v1 import Bulk_space_plot_v1  # grc-generated hier_block
from FFT_overlap_v1 import FFT_overlap_v1  # grc-generated hier_block
from NC_function_v1 import NC_function_v1  # grc-generated hier_block
from PyQt5 import Qt
from PyQt5.QtCore import QObject, pyqtSlot
from gnuradio import qtgui
import sip
from gnuradio import analog
from gnuradio import audio
from gnuradio import blocks
from gnuradio import filter
from gnuradio.filter import firdes
from gnuradio import gr
import signal
from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
from gnuradio import eng_notation
from gnuradio.qtgui import Range, RangeWidget
from phasing2xo_v1 import phasing2xo_v1  # grc-generated hier_block
from sel_block_float_v1 import sel_block_float_v1  # grc-generated hier_block

from gnuradio import qtgui

class NoiseCancelling_v1(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "NoiseCancelling_v1")
        Qt.QWidget.__init__(self)
        self.setWindowTitle("NoiseCancelling_v1")
        qtgui.util.check_set_qss()
        try:
            self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
            pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "NoiseCancelling_v1")

        try:
            if StrictVersion(Qt.qVersion()) < StrictVersion("5.0.0"):
                self.restoreGeometry(self.settings.value("geometry").toByteArray())
            else:
                self.restoreGeometry(self.settings.value("geometry"))
        except:
            pass

        ##################################################
        # Variables
        ##################################################
        self.samp_rate = samp_rate = 8000
        self.gainFile = gainFile = 0.05
        self.gainDiversity = gainDiversity = 3
        self.Threshold = Threshold = 0.25
        self.Source_sel = Source_sel = 0
        self.Ratio = Ratio = 0.0
        self.Phasing = Phasing = 0
        self.Phase = Phase = 10
        self.NC = NC = 0
        self.ABXOsel = ABXOsel = 1

        ##################################################
        # Blocks
        ##################################################
        self._gainFile_range = Range(0, 1, 0.001, 0.05, 300)
        self._gainFile_win = RangeWidget(self._gainFile_range, self.set_gainFile, 'gainFile', "counter_slider", float)
        self.top_grid_layout.addWidget(self._gainFile_win, 1, 6, 1, 5)
        for r in range(1, 2):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(6, 11):
            self.top_grid_layout.setColumnStretch(c, 1)
        self._gainDiversity_range = Range(0, 20, 0.1, 3, 300)
        self._gainDiversity_win = RangeWidget(self._gainDiversity_range, self.set_gainDiversity, 'gainDiversity', "counter_slider", float)
        self.top_grid_layout.addWidget(self._gainDiversity_win, 2, 6, 1, 5)
        for r in range(2, 3):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(6, 11):
            self.top_grid_layout.setColumnStretch(c, 1)
        self._Threshold_range = Range(0, 1.0, 0.01, 0.25, 300)
        self._Threshold_win = RangeWidget(self._Threshold_range, self.set_Threshold, 'Threshold', "counter_slider", float)
        self.top_grid_layout.addWidget(self._Threshold_win, 3, 6, 1, 5)
        for r in range(3, 4):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(6, 11):
            self.top_grid_layout.setColumnStretch(c, 1)
        # Create the options list
        self._Source_sel_options = (0, 1, )
        # Create the labels list
        self._Source_sel_labels = ('File', 'Line', )
        # Create the combo box
        # Create the radio buttons
        self._Source_sel_group_box = Qt.QGroupBox('Source_sel' + ": ")
        self._Source_sel_box = Qt.QVBoxLayout()
        class variable_chooser_button_group(Qt.QButtonGroup):
            def __init__(self, parent=None):
                Qt.QButtonGroup.__init__(self, parent)
            @pyqtSlot(int)
            def updateButtonChecked(self, button_id):
                self.button(button_id).setChecked(True)
        self._Source_sel_button_group = variable_chooser_button_group()
        self._Source_sel_group_box.setLayout(self._Source_sel_box)
        for i, _label in enumerate(self._Source_sel_labels):
            radio_button = Qt.QRadioButton(_label)
            self._Source_sel_box.addWidget(radio_button)
            self._Source_sel_button_group.addButton(radio_button, i)
        self._Source_sel_callback = lambda i: Qt.QMetaObject.invokeMethod(self._Source_sel_button_group, "updateButtonChecked", Qt.Q_ARG("int", self._Source_sel_options.index(i)))
        self._Source_sel_callback(self.Source_sel)
        self._Source_sel_button_group.buttonClicked[int].connect(
            lambda i: self.set_Source_sel(self._Source_sel_options[i]))
        self.top_grid_layout.addWidget(self._Source_sel_group_box, 4, 9, 2, 1)
        for r in range(4, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(9, 10):
            self.top_grid_layout.setColumnStretch(c, 1)
        self._Ratio_range = Range(-0.99, 0.99, 0.01, 0.0, 200)
        self._Ratio_win = RangeWidget(self._Ratio_range, self.set_Ratio, 'Ratio', "counter_slider", float)
        self.top_grid_layout.addWidget(self._Ratio_win, 5, 0, 1, 4)
        for r in range(5, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 4):
            self.top_grid_layout.setColumnStretch(c, 1)
        # Create the options list
        self._Phasing_options = (0, 1, )
        # Create the labels list
        self._Phasing_labels = ('Normal', 'Inverted', )
        # Create the combo box
        # Create the radio buttons
        self._Phasing_group_box = Qt.QGroupBox('Phasing' + ": ")
        self._Phasing_box = Qt.QVBoxLayout()
        class variable_chooser_button_group(Qt.QButtonGroup):
            def __init__(self, parent=None):
                Qt.QButtonGroup.__init__(self, parent)
            @pyqtSlot(int)
            def updateButtonChecked(self, button_id):
                self.button(button_id).setChecked(True)
        self._Phasing_button_group = variable_chooser_button_group()
        self._Phasing_group_box.setLayout(self._Phasing_box)
        for i, _label in enumerate(self._Phasing_labels):
            radio_button = Qt.QRadioButton(_label)
            self._Phasing_box.addWidget(radio_button)
            self._Phasing_button_group.addButton(radio_button, i)
        self._Phasing_callback = lambda i: Qt.QMetaObject.invokeMethod(self._Phasing_button_group, "updateButtonChecked", Qt.Q_ARG("int", self._Phasing_options.index(i)))
        self._Phasing_callback(self.Phasing)
        self._Phasing_button_group.buttonClicked[int].connect(
            lambda i: self.set_Phasing(self._Phasing_options[i]))
        self.top_grid_layout.addWidget(self._Phasing_group_box, 4, 6, 2, 1)
        for r in range(4, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(6, 7):
            self.top_grid_layout.setColumnStretch(c, 1)
        self._Phase_range = Range(0, 360, 1, 10, 200)
        self._Phase_win = RangeWidget(self._Phase_range, self.set_Phase, 'Phase', "counter_slider", float)
        self.top_grid_layout.addWidget(self._Phase_win, 4, 0, 1, 4)
        for r in range(4, 5):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 4):
            self.top_grid_layout.setColumnStretch(c, 1)
        # Create the options list
        self._NC_options = (0, 1, )
        # Create the labels list
        self._NC_labels = ('Off', 'On', )
        # Create the combo box
        # Create the radio buttons
        self._NC_group_box = Qt.QGroupBox('NC' + ": ")
        self._NC_box = Qt.QVBoxLayout()
        class variable_chooser_button_group(Qt.QButtonGroup):
            def __init__(self, parent=None):
                Qt.QButtonGroup.__init__(self, parent)
            @pyqtSlot(int)
            def updateButtonChecked(self, button_id):
                self.button(button_id).setChecked(True)
        self._NC_button_group = variable_chooser_button_group()
        self._NC_group_box.setLayout(self._NC_box)
        for i, _label in enumerate(self._NC_labels):
            radio_button = Qt.QRadioButton(_label)
            self._NC_box.addWidget(radio_button)
            self._NC_button_group.addButton(radio_button, i)
        self._NC_callback = lambda i: Qt.QMetaObject.invokeMethod(self._NC_button_group, "updateButtonChecked", Qt.Q_ARG("int", self._NC_options.index(i)))
        self._NC_callback(self.NC)
        self._NC_button_group.buttonClicked[int].connect(
            lambda i: self.set_NC(self._NC_options[i]))
        self.top_grid_layout.addWidget(self._NC_group_box, 4, 8, 2, 1)
        for r in range(4, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(8, 9):
            self.top_grid_layout.setColumnStretch(c, 1)
        # Create the options list
        self._ABXOsel_options = (0, 1, )
        # Create the labels list
        self._ABXOsel_labels = ('XO', 'AB', )
        # Create the combo box
        # Create the radio buttons
        self._ABXOsel_group_box = Qt.QGroupBox('ABXOsel' + ": ")
        self._ABXOsel_box = Qt.QVBoxLayout()
        class variable_chooser_button_group(Qt.QButtonGroup):
            def __init__(self, parent=None):
                Qt.QButtonGroup.__init__(self, parent)
            @pyqtSlot(int)
            def updateButtonChecked(self, button_id):
                self.button(button_id).setChecked(True)
        self._ABXOsel_button_group = variable_chooser_button_group()
        self._ABXOsel_group_box.setLayout(self._ABXOsel_box)
        for i, _label in enumerate(self._ABXOsel_labels):
            radio_button = Qt.QRadioButton(_label)
            self._ABXOsel_box.addWidget(radio_button)
            self._ABXOsel_button_group.addButton(radio_button, i)
        self._ABXOsel_callback = lambda i: Qt.QMetaObject.invokeMethod(self._ABXOsel_button_group, "updateButtonChecked", Qt.Q_ARG("int", self._ABXOsel_options.index(i)))
        self._ABXOsel_callback(self.ABXOsel)
        self._ABXOsel_button_group.buttonClicked[int].connect(
            lambda i: self.set_ABXOsel(self._ABXOsel_options[i]))
        self.top_grid_layout.addWidget(self._ABXOsel_group_box, 4, 7, 2, 1)
        for r in range(4, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(7, 8):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.sel_block_float_v1_0_0 = sel_block_float_v1(
            sel=NC,
        )
        self.sel_block_float_v1_0 = sel_block_float_v1(
            sel=Source_sel,
        )
        self.rational_resampler_xxx_0_1_0 = filter.rational_resampler_fff(
                interpolation=6,
                decimation=1,
                taps=None,
                fractional_bw=None)
        self.rational_resampler_xxx_0_1 = filter.rational_resampler_fff(
                interpolation=6,
                decimation=1,
                taps=None,
                fractional_bw=None)
        self.rational_resampler_xxx_0_0 = filter.rational_resampler_fff(
                interpolation=1,
                decimation=6,
                taps=None,
                fractional_bw=None)
        self.rational_resampler_xxx_0 = filter.rational_resampler_fff(
                interpolation=1,
                decimation=6,
                taps=None,
                fractional_bw=None)
        self.qtgui_number_sink_1 = qtgui.number_sink(
            gr.sizeof_float,
            0,
            qtgui.NUM_GRAPH_NONE,
            1
        )
        self.qtgui_number_sink_1.set_update_time(0.10)
        self.qtgui_number_sink_1.set_title("")

        labels = ['AX/BO (dB)', '', '', '', '',
            '', '', '', '', '']
        units = ['', '', '', '', '',
            '', '', '', '', '']
        colors = [("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"),
            ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black")]
        factor = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]

        for i in range(1):
            self.qtgui_number_sink_1.set_min(i, -60)
            self.qtgui_number_sink_1.set_max(i, 60)
            self.qtgui_number_sink_1.set_color(i, colors[i][0], colors[i][1])
            if len(labels[i]) == 0:
                self.qtgui_number_sink_1.set_label(i, "Data {0}".format(i))
            else:
                self.qtgui_number_sink_1.set_label(i, labels[i])
            self.qtgui_number_sink_1.set_unit(i, units[i])
            self.qtgui_number_sink_1.set_factor(i, factor[i])

        self.qtgui_number_sink_1.enable_autoscale(False)
        self._qtgui_number_sink_1_win = sip.wrapinstance(self.qtgui_number_sink_1.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_number_sink_1_win, 5, 4, 1, 1)
        for r in range(5, 6):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(4, 5):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_number_sink_0_0 = qtgui.number_sink(
            gr.sizeof_float,
            0,
            qtgui.NUM_GRAPH_VERT,
            1
        )
        self.qtgui_number_sink_0_0.set_update_time(0.1)
        self.qtgui_number_sink_0_0.set_title("S meter BO")

        labels = ['6dB / S point', '', '', '', '',
            '', '', '', '', '']
        units = ['', '', '', '', '',
            '', '', '', '', '']
        colors = [("blue", "red"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"),
            ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black")]
        factor = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]

        for i in range(1):
            self.qtgui_number_sink_0_0.set_min(i, 0)
            self.qtgui_number_sink_0_0.set_max(i, 20)
            self.qtgui_number_sink_0_0.set_color(i, colors[i][0], colors[i][1])
            if len(labels[i]) == 0:
                self.qtgui_number_sink_0_0.set_label(i, "Data {0}".format(i))
            else:
                self.qtgui_number_sink_0_0.set_label(i, labels[i])
            self.qtgui_number_sink_0_0.set_unit(i, units[i])
            self.qtgui_number_sink_0_0.set_factor(i, factor[i])

        self.qtgui_number_sink_0_0.enable_autoscale(False)
        self._qtgui_number_sink_0_0_win = sip.wrapinstance(self.qtgui_number_sink_0_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_number_sink_0_0_win, 0, 5, 4, 1)
        for r in range(0, 4):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(5, 6):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_number_sink_0 = qtgui.number_sink(
            gr.sizeof_float,
            0,
            qtgui.NUM_GRAPH_VERT,
            1
        )
        self.qtgui_number_sink_0.set_update_time(0.1)
        self.qtgui_number_sink_0.set_title("S meter AX")

        labels = ['6dB / S point', '', '', '', '',
            '', '', '', '', '']
        units = ['', '', '', '', '',
            '', '', '', '', '']
        colors = [("blue", "red"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"),
            ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black"), ("black", "black")]
        factor = [1, 1, 1, 1, 1,
            1, 1, 1, 1, 1]

        for i in range(1):
            self.qtgui_number_sink_0.set_min(i, 0)
            self.qtgui_number_sink_0.set_max(i, 20)
            self.qtgui_number_sink_0.set_color(i, colors[i][0], colors[i][1])
            if len(labels[i]) == 0:
                self.qtgui_number_sink_0.set_label(i, "Data {0}".format(i))
            else:
                self.qtgui_number_sink_0.set_label(i, labels[i])
            self.qtgui_number_sink_0.set_unit(i, units[i])
            self.qtgui_number_sink_0.set_factor(i, factor[i])

        self.qtgui_number_sink_0.enable_autoscale(False)
        self._qtgui_number_sink_0_win = sip.wrapinstance(self.qtgui_number_sink_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_number_sink_0_win, 0, 4, 4, 1)
        for r in range(0, 4):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(4, 5):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.qtgui_const_sink_x_0 = qtgui.const_sink_c(
            2000, #size
            "", #name
            5 #number of inputs
        )
        self.qtgui_const_sink_x_0.set_update_time(0.05)
        self.qtgui_const_sink_x_0.set_y_axis(-1, 1)
        self.qtgui_const_sink_x_0.set_x_axis(0, 360)
        self.qtgui_const_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, "")
        self.qtgui_const_sink_x_0.enable_autoscale(False)
        self.qtgui_const_sink_x_0.enable_grid(True)
        self.qtgui_const_sink_x_0.enable_axis_labels(False)

        self.qtgui_const_sink_x_0.disable_legend()

        labels = ['RX0', 'RX1', 'RX2', 'NC setting', '',
            '', '', '', '', '']
        widths = [1, 1, 1, 10, 2,
            1, 1, 1, 1, 1]
        colors = ["black", "black", "black", "red", "green",
            "red", "red", "red", "red", "red"]
        styles = [0, 0, 0, 0, 0,
            0, 0, 0, 0, 0]
        markers = [0, 0, 0, 1, 0,
            0, 0, 0, 0, 0]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0, 1.0]

        for i in range(5):
            if len(labels[i]) == 0:
                self.qtgui_const_sink_x_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_const_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_const_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_const_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_const_sink_x_0.set_line_style(i, styles[i])
            self.qtgui_const_sink_x_0.set_line_marker(i, markers[i])
            self.qtgui_const_sink_x_0.set_line_alpha(i, alphas[i])

        self._qtgui_const_sink_x_0_win = sip.wrapinstance(self.qtgui_const_sink_x_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_const_sink_x_0_win, 0, 0, 4, 4)
        for r in range(0, 4):
            self.top_grid_layout.setRowStretch(r, 1)
        for c in range(0, 4):
            self.top_grid_layout.setColumnStretch(c, 1)
        self.phasing2xo_v1_0 = phasing2xo_v1(
            sel=ABXOsel,
        )
        self.blocks_wavfile_source_0 = blocks.wavfile_source('/home/pa0sim/GNUradio38/samples/80m_SSB_NTN_stable_vyOwave.wav', True)
        self.blocks_vector_source_x_0_1_0_0 = blocks.vector_source_f([(Phase)]*512, True, 512, [])
        self.blocks_vector_source_x_0_1_0 = blocks.vector_source_f([(Ratio)]*512, True, 512, [])
        self.blocks_nlog10_ff_0_1 = blocks.nlog10_ff(20/6, 1, 2)
        self.blocks_nlog10_ff_0_0 = blocks.nlog10_ff(20, 1, 0)
        self.blocks_nlog10_ff_0 = blocks.nlog10_ff(20/6, 1, 2)
        self.blocks_multiply_const_vxx_1_1_1_1_1_0 = blocks.multiply_const_ff((gainDiversity))
        self.blocks_multiply_const_vxx_1_1_1_1_1 = blocks.multiply_const_ff((gainDiversity))
        self.blocks_multiply_const_vxx_1_1_1_1_0 = blocks.multiply_const_ff((gainFile))
        self.blocks_multiply_const_vxx_1_1_1_1 = blocks.multiply_const_ff((gainFile))
        self.blocks_multiply_const_vxx_1_1_0 = blocks.multiply_const_ff(1)
        self.blocks_multiply_const_vxx_1_1 = blocks.multiply_const_ff(1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_ff(0.001)
        self.blocks_moving_average_xx_0_0 = blocks.moving_average_ff(10000, 1, 1000, 1)
        self.blocks_moving_average_xx_0 = blocks.moving_average_ff(10000, 1, 1000, 1)
        self.blocks_float_to_complex_0 = blocks.float_to_complex(512)
        self.blocks_divide_xx_0_0 = blocks.divide_ff(1)
        self.blocks_divide_xx_0 = blocks.divide_ff(1)
        self.blocks_delay_0_0 = blocks.delay(gr.sizeof_float*1, 3300)
        self.blocks_delay_0 = blocks.delay(gr.sizeof_float*1, 3300)
        self.blocks_add_xx_0 = blocks.add_vff(1)
        self.blocks_abs_xx_0_0 = blocks.abs_ff(1)
        self.blocks_abs_xx_0 = blocks.abs_ff(1)
        self.audio_source_0_0 = audio.source(48000, '', True)
        self.audio_sink_1 = audio.sink(48000, '', True)
        self.analog_const_source_x_1_0 = analog.sig_source_f(0, analog.GR_CONST_WAVE, 0, 0, (Phase+(Phasing*180)))
        self.analog_const_source_x_1 = analog.sig_source_f(0, analog.GR_CONST_WAVE, 0, 0, ((1+Ratio)/(1-Ratio))*(1-Phasing) + ((1-Ratio)/(1+Ratio))*Phasing)
        self.NC_function_v1_0 = NC_function_v1()
        self.FFT_overlap_v1_1 = FFT_overlap_v1()
        self.FFT_overlap_v1_0 = FFT_overlap_v1()
        self.Bulk_space_plot_v1_0 = Bulk_space_plot_v1()
        self.Bulk_Space_v1_0 = Bulk_Space_v1(
            display_threshold=Threshold,
        )
        self.ABXO = ABXO(
            sel=ABXOsel,
        )



        ##################################################
        # Connections
        ##################################################
        self.connect((self.ABXO, 1), (self.Bulk_Space_v1_0, 1))
        self.connect((self.ABXO, 0), (self.Bulk_Space_v1_0, 0))
        self.connect((self.Bulk_Space_v1_0, 0), (self.Bulk_space_plot_v1_0, 1))
        self.connect((self.Bulk_space_plot_v1_0, 4), (self.qtgui_const_sink_x_0, 4))
        self.connect((self.Bulk_space_plot_v1_0, 3), (self.qtgui_const_sink_x_0, 3))
        self.connect((self.Bulk_space_plot_v1_0, 1), (self.qtgui_const_sink_x_0, 1))
        self.connect((self.Bulk_space_plot_v1_0, 2), (self.qtgui_const_sink_x_0, 2))
        self.connect((self.Bulk_space_plot_v1_0, 0), (self.qtgui_const_sink_x_0, 0))
        self.connect((self.FFT_overlap_v1_0, 0), (self.ABXO, 0))
        self.connect((self.FFT_overlap_v1_1, 0), (self.ABXO, 1))
        self.connect((self.NC_function_v1_0, 0), (self.sel_block_float_v1_0_0, 0))
        self.connect((self.NC_function_v1_0, 0), (self.sel_block_float_v1_0_0, 1))
        self.connect((self.analog_const_source_x_1, 0), (self.NC_function_v1_0, 1))
        self.connect((self.analog_const_source_x_1, 0), (self.blocks_nlog10_ff_0_0, 0))
        self.connect((self.analog_const_source_x_1_0, 0), (self.NC_function_v1_0, 0))
        self.connect((self.audio_source_0_0, 0), (self.blocks_multiply_const_vxx_1_1, 0))
        self.connect((self.audio_source_0_0, 1), (self.blocks_multiply_const_vxx_1_1_0, 0))
        self.connect((self.blocks_abs_xx_0, 0), (self.blocks_moving_average_xx_0_0, 0))
        self.connect((self.blocks_abs_xx_0_0, 0), (self.blocks_moving_average_xx_0, 0))
        self.connect((self.blocks_add_xx_0, 0), (self.blocks_multiply_const_vxx_0, 0))
        self.connect((self.blocks_delay_0, 0), (self.blocks_divide_xx_0, 0))
        self.connect((self.blocks_delay_0_0, 0), (self.blocks_divide_xx_0_0, 0))
        self.connect((self.blocks_divide_xx_0, 0), (self.audio_sink_1, 0))
        self.connect((self.blocks_divide_xx_0_0, 0), (self.audio_sink_1, 1))
        self.connect((self.blocks_float_to_complex_0, 0), (self.Bulk_space_plot_v1_0, 0))
        self.connect((self.blocks_moving_average_xx_0, 0), (self.blocks_add_xx_0, 1))
        self.connect((self.blocks_moving_average_xx_0, 0), (self.blocks_nlog10_ff_0_1, 0))
        self.connect((self.blocks_moving_average_xx_0_0, 0), (self.blocks_add_xx_0, 0))
        self.connect((self.blocks_moving_average_xx_0_0, 0), (self.blocks_nlog10_ff_0, 0))
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.blocks_divide_xx_0, 1))
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.blocks_divide_xx_0_0, 1))
        self.connect((self.blocks_multiply_const_vxx_1_1, 0), (self.sel_block_float_v1_0, 0))
        self.connect((self.blocks_multiply_const_vxx_1_1_0, 0), (self.sel_block_float_v1_0, 1))
        self.connect((self.blocks_multiply_const_vxx_1_1_1_1, 0), (self.sel_block_float_v1_0, 2))
        self.connect((self.blocks_multiply_const_vxx_1_1_1_1_0, 0), (self.sel_block_float_v1_0, 3))
        self.connect((self.blocks_multiply_const_vxx_1_1_1_1_1, 0), (self.sel_block_float_v1_0_0, 2))
        self.connect((self.blocks_multiply_const_vxx_1_1_1_1_1_0, 0), (self.sel_block_float_v1_0_0, 3))
        self.connect((self.blocks_nlog10_ff_0, 0), (self.qtgui_number_sink_0, 0))
        self.connect((self.blocks_nlog10_ff_0_0, 0), (self.qtgui_number_sink_1, 0))
        self.connect((self.blocks_nlog10_ff_0_1, 0), (self.qtgui_number_sink_0_0, 0))
        self.connect((self.blocks_vector_source_x_0_1_0, 0), (self.blocks_float_to_complex_0, 1))
        self.connect((self.blocks_vector_source_x_0_1_0_0, 0), (self.blocks_float_to_complex_0, 0))
        self.connect((self.blocks_wavfile_source_0, 1), (self.rational_resampler_xxx_0_1, 0))
        self.connect((self.blocks_wavfile_source_0, 0), (self.rational_resampler_xxx_0_1_0, 0))
        self.connect((self.phasing2xo_v1_0, 0), (self.NC_function_v1_0, 2))
        self.connect((self.phasing2xo_v1_0, 1), (self.NC_function_v1_0, 3))
        self.connect((self.phasing2xo_v1_0, 1), (self.blocks_multiply_const_vxx_1_1_1_1_1, 0))
        self.connect((self.phasing2xo_v1_0, 0), (self.blocks_multiply_const_vxx_1_1_1_1_1_0, 0))
        self.connect((self.rational_resampler_xxx_0, 0), (self.FFT_overlap_v1_1, 0))
        self.connect((self.rational_resampler_xxx_0_0, 0), (self.FFT_overlap_v1_0, 0))
        self.connect((self.rational_resampler_xxx_0_1, 0), (self.blocks_multiply_const_vxx_1_1_1_1_0, 0))
        self.connect((self.rational_resampler_xxx_0_1_0, 0), (self.blocks_multiply_const_vxx_1_1_1_1, 0))
        self.connect((self.sel_block_float_v1_0, 0), (self.phasing2xo_v1_0, 0))
        self.connect((self.sel_block_float_v1_0, 1), (self.phasing2xo_v1_0, 1))
        self.connect((self.sel_block_float_v1_0, 0), (self.rational_resampler_xxx_0, 0))
        self.connect((self.sel_block_float_v1_0, 1), (self.rational_resampler_xxx_0_0, 0))
        self.connect((self.sel_block_float_v1_0_0, 0), (self.blocks_abs_xx_0, 0))
        self.connect((self.sel_block_float_v1_0_0, 1), (self.blocks_abs_xx_0_0, 0))
        self.connect((self.sel_block_float_v1_0_0, 0), (self.blocks_delay_0, 0))
        self.connect((self.sel_block_float_v1_0_0, 1), (self.blocks_delay_0_0, 0))


    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "NoiseCancelling_v1")
        self.settings.setValue("geometry", self.saveGeometry())
        event.accept()

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate

    def get_gainFile(self):
        return self.gainFile

    def set_gainFile(self, gainFile):
        self.gainFile = gainFile
        self.blocks_multiply_const_vxx_1_1_1_1.set_k((self.gainFile))
        self.blocks_multiply_const_vxx_1_1_1_1_0.set_k((self.gainFile))

    def get_gainDiversity(self):
        return self.gainDiversity

    def set_gainDiversity(self, gainDiversity):
        self.gainDiversity = gainDiversity
        self.blocks_multiply_const_vxx_1_1_1_1_1.set_k((self.gainDiversity))
        self.blocks_multiply_const_vxx_1_1_1_1_1_0.set_k((self.gainDiversity))

    def get_Threshold(self):
        return self.Threshold

    def set_Threshold(self, Threshold):
        self.Threshold = Threshold
        self.Bulk_Space_v1_0.set_display_threshold(self.Threshold)

    def get_Source_sel(self):
        return self.Source_sel

    def set_Source_sel(self, Source_sel):
        self.Source_sel = Source_sel
        self._Source_sel_callback(self.Source_sel)
        self.sel_block_float_v1_0.set_sel(self.Source_sel)

    def get_Ratio(self):
        return self.Ratio

    def set_Ratio(self, Ratio):
        self.Ratio = Ratio
        self.analog_const_source_x_1.set_offset(((1+self.Ratio)/(1-self.Ratio))*(1-self.Phasing) + ((1-self.Ratio)/(1+self.Ratio))*self.Phasing)
        self.blocks_vector_source_x_0_1_0.set_data([(self.Ratio)]*512, [])

    def get_Phasing(self):
        return self.Phasing

    def set_Phasing(self, Phasing):
        self.Phasing = Phasing
        self._Phasing_callback(self.Phasing)
        self.analog_const_source_x_1.set_offset(((1+self.Ratio)/(1-self.Ratio))*(1-self.Phasing) + ((1-self.Ratio)/(1+self.Ratio))*self.Phasing)
        self.analog_const_source_x_1_0.set_offset((self.Phase+(self.Phasing*180)))

    def get_Phase(self):
        return self.Phase

    def set_Phase(self, Phase):
        self.Phase = Phase
        self.analog_const_source_x_1_0.set_offset((self.Phase+(self.Phasing*180)))
        self.blocks_vector_source_x_0_1_0_0.set_data([(self.Phase)]*512, [])

    def get_NC(self):
        return self.NC

    def set_NC(self, NC):
        self.NC = NC
        self._NC_callback(self.NC)
        self.sel_block_float_v1_0_0.set_sel(self.NC)

    def get_ABXOsel(self):
        return self.ABXOsel

    def set_ABXOsel(self, ABXOsel):
        self.ABXOsel = ABXOsel
        self._ABXOsel_callback(self.ABXOsel)
        self.ABXO.set_sel(self.ABXOsel)
        self.phasing2xo_v1_0.set_sel(self.ABXOsel)





def main(top_block_cls=NoiseCancelling_v1, options=None):

    if StrictVersion("4.5.0") <= StrictVersion(Qt.qVersion()) < StrictVersion("5.0.0"):
        style = gr.prefs().get_string('qtgui', 'style', 'raster')
        Qt.QApplication.setGraphicsSystem(style)
    qapp = Qt.QApplication(sys.argv)

    tb = top_block_cls()

    tb.start()

    tb.show()

    def sig_handler(sig=None, frame=None):
        Qt.QApplication.quit()

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    timer = Qt.QTimer()
    timer.start(500)
    timer.timeout.connect(lambda: None)

    def quitting():
        tb.stop()
        tb.wait()

    qapp.aboutToQuit.connect(quitting)
    qapp.exec_()

if __name__ == '__main__':
    main()
